@@ -154,6 +154,11 @@ class TestLinker(tests.TestCase):
 	def runTest(self):
 		dir = Dir(self.get_tmp_name())
 		notebook = tests.new_notebook(fakedir=dir.subdir('notebook'))
+		for name in ('foo', 'bar', 'foo:bar',):
+			p = notebook.get_page(Path(name))
+			p.parse('wiki', "test 123")
+			notebook.store_page(p)
+
 		layout = MultiFileLayout(dir.subdir('layout'), 'html')
 		source = Path('foo:bar')
 		output = layout.page_file(source)
@@ -361,7 +366,7 @@ class TestPageSelections(tests.TestCase):
 class TestMultiFileExporter(tests.TestCase):
 
 	def runTest(self):
-		dir =  Dir(self.create_tmp_dir())
+		dir = Dir(self.create_tmp_dir())
 		#~ dir =  VirtualDir('/test')
 		notebook = tests.new_notebook(fakedir='/foo')
 		pages = AllPages(notebook)
@@ -371,18 +376,18 @@ class TestMultiFileExporter(tests.TestCase):
 		exporter.export(pages)
 
 		file = exporter.layout.page_file(Path('roundtrip'))
-		text =  file.read()
+		text = file.read()
 		self.assertIn('Lorem ipsum dolor sit amet', text)
 
 		file = exporter.layout.page_file(Path('Index'))
-		text =  file.read()
+		text = file.read()
 		self.assertIn('<li><a href="./roundtrip.html" title="roundtrip" class="page">roundtrip</a></li>', text)
 
 
 class TestSingleFileExporter(tests.TestCase):
 
 	def runTest(self):
-		dir =  Dir(self.create_tmp_dir())
+		dir = Dir(self.create_tmp_dir())
 		#~ dir =  VirtualDir('/test')
 		file = dir.file('export.html')
 		notebook = tests.new_notebook(fakedir='/foo')
@@ -392,7 +397,7 @@ class TestSingleFileExporter(tests.TestCase):
 		self.assertIsInstance(exporter, SingleFileExporter)
 		exporter.export(pages)
 
-		text =  file.read()
+		text = file.read()
 		self.assertIn('Lorem ipsum dolor sit amet', text)
 
 
@@ -400,7 +405,7 @@ class TestSingleFileExporter(tests.TestCase):
 class TestMHTMLExporter(tests.TestCase):
 
 	def runTest(self):
-		dir =  Dir(self.create_tmp_dir())
+		dir = Dir(self.create_tmp_dir())
 		#~ dir =  VirtualDir('/test')
 		file = dir.file('export.mht')
 		notebook = tests.new_notebook(fakedir='/foo')
@@ -410,14 +415,14 @@ class TestMHTMLExporter(tests.TestCase):
 		self.assertIsInstance(exporter, MHTMLExporter)
 		exporter.export(pages)
 
-		text =  file.read()
+		text = file.read()
 		self.assertIn('Lorem ipsum dolor sit amet', text)
 
 
 class TestTemplateOptions(tests.TestCase):
 
 	def runTest(self):
-		dir =  Dir(self.create_tmp_dir())
+		dir = Dir(self.create_tmp_dir())
 		file = dir.file('test.tex')
 		page = Path('roundtrip')
 		exporter = build_page_exporter(file, 'latex', 'Article', page)
@@ -436,7 +441,7 @@ class TestTemplateOptions(tests.TestCase):
 class TestExportFormat(object):
 
 	def runTest(self):
-		dir =  Dir(self.create_tmp_dir())
+		dir = Dir(self.create_tmp_dir())
 		#~ dir =  VirtualDir('/test')
 
 		i = 0
@@ -452,7 +457,7 @@ class TestExportFormat(object):
 				exporter.export(pages)
 
 			file = exporter.layout.page_file(Path('roundtrip'))
-			text =  file.read()
+			text = file.read()
 			self.assertIn('Lorem ipsum dolor sit amet', text)
 
 			i += 1
@@ -582,6 +587,20 @@ class TestExportCommand(tests.TestCase):
 		self.assertIsNotNone(exp.format)
 		self.assertIsNotNone(exp.index_page)
 
+
+		## Full notebook, single page
+		cmd = ExportCommand('export')
+		cmd.parse_options(self.notebook.path,
+			'--format', 'markdown',
+			'--template', './tests/data/TestTemplate.html',
+			'--output', self.tmpdir.file('output.md').path,
+			'-s'
+		)
+		exp = cmd.get_exporter(None)
+		self.assertIsInstance(exp, SingleFileExporter)
+		self.assertIsInstance(exp.layout, SingleFileLayout)
+		self.assertIsInstance(exp.layout.file, File)
+
 		## Single page
 		cmd = ExportCommand('export')
 		cmd.parse_options(self.notebook.path, 'Foo:Bar',
