@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright 2008-2014 Jaap Karssenberg <jaap.karssenberg@gmail.com>
+# Copyright 2008-2018 Jaap Karssenberg <jaap.karssenberg@gmail.com>
 
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -63,9 +63,6 @@ can be extended by plugins.
 The graphical user interface is implemented in the L{zim.gui} module
 and it's sub-modules. The webinterface is implemented in L{zim.www}.
 
-The graphical interface uses a background process to coordinate
-between running instances, this is implemented in L{zim.ipc}.
-
 Functionality for exporting content is implemented in L{zim.exporter}.
 And search functionality can be found in L{zim.search}.
 
@@ -99,11 +96,11 @@ Some generic base classes and functions can be found in L{zim.utils}
 
 
 # Bunch of meta data, used at least in the about dialog
-__version__ = '0.63'
-__url__='http://www.zim-wiki.org'
+__version__ = '0.69.1'
+__url__ = 'http://www.zim-wiki.org'
 __author__ = 'Jaap Karssenberg <jaap.karssenberg@gmail.com>'
-__copyright__ = 'Copyright 2008 - 2014 Jaap Karssenberg <jaap.karssenberg@gmail.com>'
-__license__='''\
+__copyright__ = 'Copyright 2008 - 2018 Jaap Karssenberg <jaap.karssenberg@gmail.com>'
+__license__ = '''\
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
@@ -124,10 +121,6 @@ import locale
 logger = logging.getLogger('zim')
 
 
-#: This parameter can be set by ./setup.py, can be e.g. "maemo"
-PLATFORM = None
-
-
 ########################################################################
 
 ## Note: all init here must happen before importing any other zim
@@ -160,21 +153,26 @@ if os.name == "nt" and not os.environ.get('LANG'):
 	# Set locale config for gettext (other platforms have this by default)
 	# Using LANG because it is lowest prio - do not override other params
 	lang, enc = locale.getlocale()
-	os.environ['LANG'] = lang + '.' + enc
-	logging.info('Locale set to: %s', os.environ['LANG'])
+	if lang is not None:
+		os.environ['LANG'] = lang + '.' + enc if enc else lang
+		logger.info('Locale set to: %s', os.environ['LANG'])
 
 
 _localedir = os.path.join(os.path.dirname(ZIM_EXECUTABLE), 'locale')
 if not os.name == "nt":
 	_localedir = _localedir.encode(sys.getfilesystemencoding())
 
-if os.path.isdir(_localedir):
-	# We are running from a source dir - use the locale data included there
-	gettext.install('zim', _localedir, unicode=True, names=('_', 'gettext', 'ngettext'))
-else:
-	# Hope the system knows where to find the data
-	gettext.install('zim', None, unicode=True, names=('_', 'gettext', 'ngettext'))
-
+try:
+	if os.path.isdir(_localedir):
+		# We are running from a source dir - use the locale data included there
+		gettext.install('zim', _localedir, unicode=True, names=('_', 'gettext', 'ngettext'))
+	else:
+		# Hope the system knows where to find the data
+		gettext.install('zim', None, unicode=True, names=('_', 'gettext', 'ngettext'))
+except:
+	logger.exception('Error loading translation')
+	trans = gettext.NullTranslations()
+	trans.install(unicode=True, names=('_', 'gettext', 'ngettext'))
 
 
 
